@@ -73,7 +73,6 @@ typedef struct {
 } EmpathyNewChatroomDialog;
 
 enum {
-	COL_IMAGE,
 	COL_NAME,
 	COL_ROOM,
 	COL_COUNT
@@ -185,6 +184,9 @@ empathy_new_chatroom_dialog_show (GtkWindow *parent)
 
 	/* Account chooser for custom */
 	dialog->account_chooser = empathy_account_chooser_new ();
+	empathy_account_chooser_set_filter (EMPATHY_ACCOUNT_CHOOSER (dialog->account_chooser),
+					    empathy_account_chooser_filter_is_connected,
+					    NULL);
 	gtk_table_attach_defaults (GTK_TABLE (dialog->table_info),
 				   dialog->account_chooser,
 				   1, 3, 0, 1);
@@ -274,13 +276,6 @@ new_chatroom_dialog_model_add_columns (EmpathyNewChatroomDialog *dialog)
 	view = GTK_TREE_VIEW (dialog->treeview);
 	gtk_tree_view_set_headers_visible (view, FALSE);
 
-	/* Chatroom pointer */
-	column = gtk_tree_view_column_new ();
-	gtk_tree_view_column_set_title (column, _("Chat Rooms"));
-
-	cell = gtk_cell_renderer_pixbuf_new ();
-	gtk_tree_view_column_pack_start (column, cell, FALSE);
-
 	cell = gtk_cell_renderer_text_new ();
 	g_object_set (cell,
 		      "xpad", (guint) 4,
@@ -288,7 +283,10 @@ new_chatroom_dialog_model_add_columns (EmpathyNewChatroomDialog *dialog)
 		      "ellipsize", PANGO_ELLIPSIZE_END,
 		      NULL);
 
-	gtk_tree_view_column_pack_start (column, cell, TRUE);
+	column = gtk_tree_view_column_new_with_attributes (_("Chat Rooms"),
+		                                           cell,
+		                                           "text", COL_NAME,
+		                                           NULL);
 
 	gtk_tree_view_column_set_expand (column, TRUE);
 	gtk_tree_view_append_column (view, column);
@@ -316,7 +314,7 @@ new_chatroom_dialog_update_widgets (EmpathyNewChatroomDialog *dialog)
 		gtk_widget_show (dialog->vbox_browse);
 
 	}
-	else if (strcmp (protocol, "salut") == 0) {
+	else if (strcmp (protocol, "local-xmpp") == 0) {
 		gtk_widget_set_sensitive (dialog->entry_server, FALSE);
 		gtk_widget_show (dialog->vbox_browse);		
 	}
@@ -475,7 +473,6 @@ new_chatroom_dialog_model_selection_changed (GtkTreeSelection         *selection
 	gtk_entry_set_text (GTK_ENTRY (dialog->entry_server), server ? server : "");
 	gtk_entry_set_text (GTK_ENTRY (dialog->entry_room), room ? room : "");
 
-	g_free (server);
 	g_free (room);
 }
 
